@@ -26,7 +26,7 @@ from matplotlib.ticker import FuncFormatter
 
 import atomica
 import sciris as sc
-from .model import Compartment, Characteristic, Parameter, Link, TimedLink, SourceCompartment, JunctionCompartment, SinkCompartment
+from .model import Compartment, Characteristic, Parameter, Link, SourceCompartment, JunctionCompartment, SinkCompartment
 from .results import Result
 from .system import logger, NotFoundError
 from .function_parser import parse_function
@@ -44,9 +44,9 @@ settings["dpi"] = 150  # average quality
 settings["transparent"] = False
 
 
-def save_figs(figs, path=".", prefix="", fnames=None) -> None:
+def save_figs(figs, path=".", prefix="", fnames=None, file_format="png") -> None:
     """
-    Save figures to disk as PNG files
+    Save figures to disk as PNG or other graphics format files
 
     Functions like `plot_series` and `plot_bars` can generate multiple figures, depending on
     the data and legend options. This function facilitates saving those figures together.
@@ -63,7 +63,7 @@ def save_figs(figs, path=".", prefix="", fnames=None) -> None:
     a legend and will be named based on the name of the figure immediately before it.
     If you provide an empty string in the `fnames` argument this same operation will be carried
     out. If the last figure name is omitted, an empty string will automatically be added.
-
+    :param file_format: the file format to save as, default png, allowed formats {png, ps, pdf, svg}
     """
 
     try:
@@ -88,6 +88,8 @@ def save_figs(figs, path=".", prefix="", fnames=None) -> None:
     assert len(fnames) == len(figs), "Number of figures must match number of specified filenames, or the last figure must be a legend with no label"
     assert fnames[0], "The first figure name cannot be empty"
 
+    assert file_format in ["png", "ps", "pdf", "svg"], f'File format {file_format} invalid. Format must be one of "png", "ps", "pdf", or "svg"'
+
     for i, fig in enumerate(figs):
         if not fnames[i]:  # assert above means that i>0
             fnames[i] = fnames[i - 1] + "_legend"
@@ -97,7 +99,7 @@ def save_figs(figs, path=".", prefix="", fnames=None) -> None:
             bbox = legend.get_window_extent().transformed(fig.dpi_scale_trans.inverted())
         else:
             bbox = "tight"
-        fname = prefix + fnames[i] + ".png"
+        fname = prefix + fnames[i] + "." + file_format
         fname = sc.sanitizefilename(fname)  # parameters may have inappropriate characters
         fig.savefig(os.path.join(path, fname), bbox_inches=bbox, dpi=settings["dpi"], transparent=settings["transparent"])
         logger.info('Saved figure "%s"', fname)
@@ -329,7 +331,7 @@ class PlotData:
                             continue
 
                         units = list(set([output_units[x] for x in labels]))
-                        timescales = list(set([output_timescales[x] for x in labels]))
+                        timescales = list(set([np.nan if isna(output_timescales[x]) else output_timescales[x] for x in labels]))  # Ensure that None and nan don't appear as different timescales
 
                         # Set default aggregation method depending on the units of the quantity
                         if output_aggregation is None:
@@ -481,8 +483,8 @@ class PlotData:
         time aggregation can be chained with other operations, the same as `PlotData.interpolate()`.
 
         :param t_bins: Vector of bin edges OR a scalar bin size, which will be automatically expanded to a vector of bin edges
-        :param time_aggregation: can be 'sum' or 'average'. Note that for quantities that have a timescale, 'sum' behaves like integration
-                                 so flow parameters in number units will be adjusted accordingly (e.g. a parameter in units of 'people/day'
+        :param time_aggregation: can be 'integrate' or 'average'. Note that for quantities that have a timescale, flow parameters
+                                 in number units will be adjusted accordingly (e.g. a parameter in units of 'people/day'
                                  aggregated over a 1 year period will display as the equivalent number of people that year)
         :param interpolation_method: Assumption on how the quantity behaves in between timesteps - in general, 'linear' should be suitable for
                                      most dynamic quantities, while 'previous' should be used for spending and other program-related quantities.
@@ -846,6 +848,8 @@ class PlotData:
             plotdata.outputs[key] = results[0].model.progset.programs[key].label if key in results[0].model.progset.programs else key
 
         if t_bins is not None:
+            # TODO - time aggregation of coverage_number by integration should only be applied to one-off programs
+            # TODO - confirm time aggregation of spending is correct for the units entered in databook or in overwrites
             if quantity in {"spending", "equivalent_spending", "coverage_number"}:
                 plotdata.time_aggregate(t_bins, "integrate", interpolation_method="previous")
             elif quantity in {"coverage_eligible", "coverage_fraction"}:
@@ -1343,7 +1347,7 @@ def plot_bars(plotdata, stack_pops=None, stack_outputs=None, outer=None, legend_
         ax.set_yticks([x[0] for x in block_labels])
         ax.set_yticklabels([x[1] for x in block_labels])
         ax.invert_yaxis()
-        ax.xaxis.set_major_formatter(FuncFormatter(sc.SItickformatter))
+        sc.SIticks(ax=ax, axis="x")
     else:
         ax.set_xlim(left=-2 * gaps[0], right=block_offset + base_offset)
         fig.set_figwidth(1.1 + 1.1 * (block_offset + base_offset))
@@ -1354,7 +1358,7 @@ def plot_bars(plotdata, stack_pops=None, stack_outputs=None, outer=None, legend_
             ax.spines["top"].set_position("zero")
         ax.set_xticks([x[0] for x in block_labels])
         ax.set_xticklabels([x[1] for x in block_labels])
-        ax.yaxis.set_major_formatter(FuncFormatter(sc.SItickformatter))
+        sc.SIticks(ax=ax, axis="y")
 
     # Calculate the units. As all bar patches are shown on the same axis, they are all expected to have the
     # same units. If they do not, the plot could be misleading
@@ -1635,7 +1639,7 @@ def _apply_series_formatting(ax, plot_type) -> None:
         ax.set_ylabel("Proportion " + ax.get_ylabel())
     else:
         ax.set_ylim(top=ax.get_ylim()[1] * 1.05)
-    ax.yaxis.set_major_formatter(FuncFormatter(sc.SItickformatter))
+    sc.SIticks(ax=ax, axis="y")
 
 
 def _turn_off_border(ax) -> None:
