@@ -12,7 +12,7 @@ from datetime import timezone
 
 import numpy as np
 import xlsxwriter as xw
-from numpy import inf, array, exp, minimum
+from numpy import inf, array, exp
 from xlsxwriter.utility import xl_rowcol_to_cell as xlrc
 
 import sciris as sc
@@ -44,6 +44,8 @@ class ProgramInstructions:
     - Spending (in units of people/year)
     - Capacity (in units of people/year)
     - Fraction/proportion coverage
+        - For continuous programs, specified as a dimensionless fraction
+        - For one-off programs, specified as a 'fraction/year' coverage
 
     which thus provides the underlying implementation for program-related scenarios. The
     :class:`ProgramSet` and :class:`Program` methods access and use the :class:`ProgramInstructions`
@@ -79,7 +81,9 @@ class ProgramInstructions:
     :param capacity: Overwrites to capacity. This is a dict keyed by program name, containing a scalar capacity or a TimeSeries of capacity values
                      For convenience, the capacity overwrite should be in units of 'people/year' and it will be automatically
                      converted to a per-timestep value based on the units for the program's unit cost.
-    :param coverage: Overwrites to proportion coverage. This is a dict keyed by program name, containing a scalar coverage or a TimeSeries of coverage values
+    :param coverage: Overwrites to proportion coverage. This is a dict keyed by program name, containing a scalar coverage or a TimeSeries
+                     of coverage values. The overwrite is specified in dimensionless units for continuous programs, and '/year' units for
+                     one-off programs.
 
     """
 
@@ -445,8 +449,8 @@ class ProgramSet(NamedItem):
 
         return framework, data
 
-    @staticmethod
-    def from_spreadsheet(spreadsheet=None, framework=None, data=None, project=None, name=None, _allow_missing_data=False):
+    @classmethod
+    def from_spreadsheet(cls, spreadsheet=None, framework=None, data=None, project=None, name=None, _allow_missing_data=False):
         """
         Instantiate a ProgramSet from a spreadsheet
 
@@ -466,10 +470,10 @@ class ProgramSet(NamedItem):
         """
         import openpyxl
 
-        framework, data = ProgramSet._normalize_inputs(framework, data, project)
+        framework, data = cls._normalize_inputs(framework, data, project)
 
         # Populate the available pops, comps, and pars based on the framework and data provided at this step
-        self = ProgramSet(name=name, framework=framework, data=data)
+        self = cls(name=name, framework=framework, data=data)
 
         # Create and load spreadsheet
         if not isinstance(spreadsheet, sc.Spreadsheet):
@@ -630,7 +634,7 @@ class ProgramSet(NamedItem):
             widths[col] = 12  # Wrap population names
 
         for comp, spec in comps_to_write.items():
-            if spec["non_targetable"] and not comp in comps_in_use:
+            if spec["non_targetable"] and comp not in comps_in_use:
                 continue
             col = comp_col[comp]
             sheet.write(1, col, spec["label"], self._formats["rc_title"]["left"]["T"])
@@ -1040,14 +1044,14 @@ class ProgramSet(NamedItem):
             else:
                 capacities[prog.name] = instructions.capacity[prog.name].interpolate(tvec, method="previous")
                 # Capacity overwrites are input in units of people/year so convert to units of people here
-                if "/year" not in prog.unit_cost.units:
+                if prog.is_one_off:
                     capacities[prog.name] *= dt
 
         return capacities
 
-    def get_prop_coverage(self, tvec, capacities: dict, num_eligible: dict, dt: float, instructions=None) -> dict:
+    def get_prop_coverage(self, tvec, dt, capacities: dict, num_eligible: dict, instructions=None) -> dict:
         """
-        Return fractional coverage
+        Return dimensionless (timestep) fractional coverage
 
         Note that this function is primarily for internal usage (i.e. during
         model integration or reconciliation). Since the proportion covered depends
@@ -1061,29 +1065,32 @@ class ProgramSet(NamedItem):
         - instructions can override the coverage (for coverage scenarios)
         - Programs can contain saturation constraints
 
+        Note that while converage overwrites might be specified in '/year' units for one-off programs,
+        *this function always returns dimensionless coverage* (i.e. not coverage/year). This coverage is also capped at 1.
+
         :param tvec: scalar year, or array of years - this is required to interpolate time-varying saturation values
         :param capacities: dict of program coverages, should match the available programs (typically the output of ``ProgramSet.get_capacities()``)
                            Note that since the capacity and eligible compartment sizes are being compared here,
                            the capacity needs to be in units of 'people' (not 'people/year') at this point
         :param num_eligible: dict of number of people covered by each program, computed externally and with one entry for each program
-        :param dt: simulation timestep
         :param instructions: optionally specify instructions, which can supply a coverage overwrite
         :return: Dict like ``{prog_name: np.array()}`` with fractional coverage values (dimensionless)
 
         """
 
         prop_coverage = sc.odict()  # Initialise outputs
+
         for prog in self.programs.values():
             if instructions is None or prog.name not in instructions.coverage:
+                # The capacities have already been converted to timestep values, so no further transformation is necessary
                 prop_coverage[prog.name] = prog.get_prop_covered(tvec, capacities[prog.name], num_eligible[prog.name])
             else:
                 prop_coverage[prog.name] = instructions.coverage[prog.name].interpolate(tvec, method="previous")
                 if prog.is_one_off:
-                    # Scale by dt beforehand, so that a timestep coverage of 1 can be achieved with an annual coverage
-                    # greater than 1 - that is, the number of people covered in a year relative to the CURRENT
-                    # compartment size
+                    # Coverage overwrites for one off programs are specified in /year units, therefore they get adjusted by dt here
                     prop_coverage[prog.name] *= dt
-                prop_coverage[prog.name] = minimum(prop_coverage[prog.name], 1.0)
+
+            prop_coverage[prog.name] = np.minimum(prop_coverage[prog.name], 1.0)
 
         return prop_coverage
 
@@ -1262,9 +1269,9 @@ class Program(NamedItem):
         spending = sc.promotetoarray(spending)
 
         unit_cost = self.unit_cost.interpolate(tvec, method="previous")
-        if "/year" not in self.unit_cost.units:
-            # The spending is $/year, and the /year gets eliminated if the unit cost is also per year. If that's not the case, then
-            # we need to multiply the spending by the timestep to get the correct units
+        if self.is_one_off:
+            # The spending is $/year, and the /year gets eliminated if the unit cost is also per year. For one-off programs, the unit cost is not
+            # /year, therefore we need to multiply the spending by the timestep to the capacity as people rather than people/year
             spending *= dt
 
         capacity = spending / unit_cost
@@ -1304,7 +1311,7 @@ class Program(NamedItem):
             prop_covered = np.divide(capacity, eligible, out=np.full(capacity.shape, np.inf), where=eligible != 0)
             saturation = self.saturation.interpolate(tvec, method="previous")
             prop_covered = 2 * saturation / (1 + exp(-2 * prop_covered / saturation)) - saturation
-            prop_covered = minimum(prop_covered, 1.0)  # Ensure that coverage doesn't go above 1 (if saturation is < 1)
+            prop_covered = np.minimum(prop_covered, 1.0)  # Ensure that coverage doesn't go above 1 (if saturation is < 1)
         else:
             # The division below means that 0/0 is treated as returning 1
             prop_covered = np.divide(capacity, eligible, out=np.ones_like(capacity), where=eligible > capacity)
@@ -1443,7 +1450,8 @@ class Covout:
                              programs in `self.progs`. Note that `coverage` is expected to be a ``np.array``
                              (such that that generated by :meth:`ProgramSet.get_prop_coverage`). However,
                              because the modality calculations only work for scalars, only the first entry
-                             in the array will be used.
+                             in the array will be used. Note that the coverage in this dictionary should be
+                             dimensionless.
         :return: A scalar outcome (of type `np.double` or similar i.e. _not_ an array)
 
         """
