@@ -250,7 +250,9 @@ class Result(NamedItem):
             output = capacities
         else:
             # Get the program coverage denominator
-            num_eligible = dict()  # This is the coverage denominator, number of people covered by the program
+            num_eligible = defaultdict(float)  # This is the coverage denominator, number of people covered by the program
+            # It is a defaultdict because if a virtual product doesn't target a population (due to a population type mismatch) then it won't have an
+            # entry in the dictionary. Setting it to zero here means the number covered will come out as zero rather than a key error
             for prog in self.model.progset.programs.values():  # For each program
                 for pop_name in prog.target_pops:
                     for comp_name in prog.target_comps:
@@ -269,7 +271,7 @@ class Result(NamedItem):
             # Note that `ProgramSet.get_prop_coverage()` takes in capacity in units of 'people' which matches
             # the units of 'num_eligible' so we therefore use the returned value from `ProgramSet.get_capacities()`
             # as-is without doing any annualization
-            prop_coverage = self.model.progset.get_prop_coverage(tvec=self.t, capacities=capacities, num_eligible=num_eligible, dt=self.dt, instructions=self.model.program_instructions)
+            prop_coverage = self.model.progset.get_prop_coverage(tvec=self.t, dt=self.dt, capacities=capacities, num_eligible=num_eligible, instructions=self.model.program_instructions)
 
             if quantity in {"fraction", "annual_fraction"}:
                 output = prop_coverage
