@@ -531,7 +531,7 @@ class TimeSeries:
 
         so for example, if you wanted to use the base Scipy pchip method with no extrapolation, then could pass in
 
-        >>> TimeSeries.interpolate(...,method=scipy.interpolate.PchipInterpolator)
+            TimeSeries.interpolate(...,method=scipy.interpolate.PchipInterpolator)
 
         Note that the following special behaviours apply:
 
@@ -877,7 +877,10 @@ def parallel_progress(fcn, inputs, num_workers=None, show_progress=True) -> list
 
     """
 
-    from multiprocessing import pool
+    from multiprocessing import pool, cpu_count
+
+    if num_workers is None:
+        num_workers = min(cpu_count(), len(inputs))
 
     pool = pool.Pool(num_workers, initializer=_worker_init)
 
@@ -894,16 +897,20 @@ def parallel_progress(fcn, inputs, num_workers=None, show_progress=True) -> list
         if show_progress:
             pbar.update(1)
 
+    jobs = []
     if sc.isnumber(inputs):
         for i in range(inputs):
-            pool.apply_async(fcn, callback=partial(callback, idx=i))
+            jobs.append(pool.apply_async(fcn, callback=partial(callback, idx=i)))
     else:
         for i, x in enumerate(inputs):
-            pool.apply_async(fcn, args=(x,), callback=partial(callback, idx=i))
+            jobs.append(pool.apply_async(fcn, args=(x,), callback=partial(callback, idx=i)))
 
     pool.close()
     pool.join()
 
+    for job in jobs:
+        job.get()  # This will raise any exceptions thrown from within the async task
+
     if show_progress:
         pbar.close()
 
